<?php

declare(strict_types = 1);

namespace Drupal\meta_entity\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the meta entity entities edit forms.
 */
class MetaEntityForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $result = parent::save($form, $form_state);

    $entity = $this->getEntity();
    $link = $entity->toLink($this->t('View'))->toRenderable();

    $message_arguments = ['%label' => $this->getEntity()->label()];
    $logger_arguments = $message_arguments + ['link' => render($link)];

    if ($result === SAVED_NEW) {
      $this->messenger()->addStatus($this->t('New meta entity %label has been created.', $message_arguments));
      $this->logger('meta_entity')->notice('Created meta new entity %label', $logger_arguments);
    }
    else {
      $this->messenger()->addStatus($this->t('The %label meta entity has been updated.', $message_arguments));
      $this->logger('meta_entity')->notice('Updated %label meta entity meta.', $logger_arguments);
    }

    $form_state->setRedirect('entity.meta_entity.canonical', [
      'meta_entity' => $entity->id(),
    ]);

    return $result;
  }

}
